#include <avr/pgmspace.h> // модуль для работы с памятью программ
#include <avr/power.h>    // модуль для изменения тактовой частоты Ардуино
#include <Wire.h>
#include <LiquidCrystal_I2C_Ext.h>
LiquidCrystal_I2C_Ext lcd(0x27, 20, 4);

// Пины для энкодера
#define pinCLK 2
#define pinDT  3
#define pinSW  4

// Перечислим значения, используемые в меню для задания связи родитель-потомок
enum {mkBack, mkRoot, mkNewFunctions, mkPrintAt, mkPrintf, mkPrintfAt, mkPrintMultiline, mkInputIntVal,
      mkInputIntValAt, mkInputLongVal, mkInputStrVal, mkSelectVal, mkShowMenu, mkAFewExamples, mkExampleSum, 
      mkExampleArray, mkExampleSelectVal, mkExampleInputStrVal, mkAbout
     };

// Опишем меню
sMenuItem menu[] = {
  {mkBack, mkRoot, "Main menu"},
    {mkRoot, mkNewFunctions, "New functions"},
      {mkNewFunctions, mkPrintAt, "printAt"},
      {mkNewFunctions, mkPrintf, "printf"},
      {mkNewFunctions, mkPrintfAt, "printfAt"},
      {mkNewFunctions, mkPrintMultiline, "printMultiline"},
      {mkNewFunctions, mkInputIntVal, "inputIntVal"},
      {mkNewFunctions, mkInputIntValAt, "inputIntValAt"},
      {mkNewFunctions, mkInputLongVal, "inputLongVal"},
      {mkNewFunctions, mkInputStrVal, "inputStrVal"},
      {mkNewFunctions, mkSelectVal, "selectVal"},
      {mkNewFunctions, mkShowMenu, "showMenu"},
      {mkNewFunctions, mkBack, "Back"},
    {mkRoot, mkAFewExamples, "A few examples"},
      {mkAFewExamples, mkExampleSum, "A + B = ?"},
      {mkAFewExamples, mkExampleArray, "inputIntValAt example"},
      {mkAFewExamples, mkExampleSelectVal, "selectVal example"},
      {mkAFewExamples, mkExampleInputStrVal, "inputStrVal example"},
      {mkAFewExamples, mkBack, "Back"},
    {mkRoot, mkAbout, "About"}
};

// Определим колчество элементов в меню
uint8_t menuLen = sizeof(menu) / sizeof(sMenuItem);

// Поместим несколько строк (краткое описание новых функций) в память программ
const char infoPrintAt[] PROGMEM = "printAt combines the functionality of setCursor and print";
const char infoPrintf[] PROGMEM = "With printf you canprint formatted data directly to lcd. Buffers and sprintfare no longer needed!";
const char infoPrintfAt[] PROGMEM = "As you might have guessed, printfAt allows you to print formatted data to the specified position on the lcd";
const char infoPrintMultiline[] PROGMEM = "printMultiline allows you to print long text with the ability to scroll. The text you are reading now is displayed by this function.";
const char infoInputIntVal[] PROGMEM = "Use inputIntVal to enter integer values with rotary encoder. It is better to use it for small values.";
const char infoInputIntValAt[] PROGMEM = "The same as a inputIntVal, but whitout a title. And you can specify a position for input.";
const char infoInputLongVal[] PROGMEM = "With inputLongVal you can enter a value by editing individual digits. Allows you to enter large values.";
const char infoInputStrVal[] PROGMEM = "inputStrVal works like inputLongVal, but you can also enter letters";
const char infoSelectVal[] PROGMEM = "selectVal allows to select value from a list. In some cases usng this function is more convenient than using the showMenu.";
const char infoShowMenu[] PROGMEM = "Shows menu and returns selected item";

// Объявим массив для удобства работы со строками в памяти программ
const char* const aboutText[] PROGMEM = {infoPrintAt, infoPrintf, infoPrintfAt, infoPrintMultiline, infoInputIntVal,
                                         infoInputIntValAt, infoInputLongVal, infoInputStrVal, infoSelectVal, infoShowMenu
                                        };

void setup() {
  lcd.begin();
  lcd.attachEncoder(pinDT, pinCLK, pinSW);
}

void loop() {
  char *buffer; // Буфер для чтения строк из FLASH
  uint8_t selectedMenuItem = lcd.showMenu(menu, menuLen, 1); // Показываем меню
  //и анализируем возвращенное значение в переменной selectedMenuItem
  if ((selectedMenuItem >= mkPrintAt) and (selectedMenuItem <= mkShowMenu)) {
    // Выбрали один из пунктов меню New functions. Покажем информацию о выбранной функции
    // Для этого выделим память, достаточную для хранения самой длинной нашей строки
    buffer = (char*) malloc(150);
    // поместим в нее данные из памяти программ
    strcpy_P(buffer, (char*)pgm_read_word(&(aboutText[selectedMenuItem - mkPrintAt])));
    lcd.printMultiline(buffer); // и выведем на дисплей функцей printMultiline
    free(buffer);
    return;
  }
  else if (selectedMenuItem == mkExampleSum)
    exampleSum(); 
  else if (selectedMenuItem == mkExampleArray)
    exampleArray();
  else if (selectedMenuItem == mkExampleSelectVal)
    exampleSelectVal();
  else if (selectedMenuItem == mkExampleInputStrVal)
    exampleInputStrVal();
  else if (selectedMenuItem == mkAbout){
    lcd.printMultiline(F("LiquidCrystal_I2C_Ext library. tsibrov.blogspot.com"));
    return;
  }
  lcd.printAt(0, 2, "Press button");
  lcd.printAt(0, 3, "to continue");
  while (lcd.getEncoderState() != eButton); // Для продолжения ждем нажатия кнопки
}

void exampleSum(){
  // Запросм 2 целых числа и выведем их сумму для демонстрации функций inputIntVal и printfAt
  int a, b;
  a = lcd.inputVal("Input A", 0, 10, 0); // title, value, min, max
  b = lcd.inputVal("Input B", 0, 10, 0); // title, value, min, max
  lcd.printfAt(0, 0, "%d + %d = %d", a, b, a + b);
}

void exampleArray(){
  // Запросм 10 целых чисел и выведем минимльное, максимальное и среднее значения
  int mas[10];
  int _min = 10, _max = -10, _avg = 0;
  lcd.printMultiline(F("This example shows how you can use inputIntValAt function to query values in a loop"));
  for (int i = 0; i < 10; i++) { // В цикле запрашиваем значения
    lcd.printfAt(0, 0, "Input A[%d]:    ", i);
    mas[i] = lcd.inputValAt(13, 0, -10, 10, 0);
    if (mas[i] < _min) _min = mas[i];  // Находим минимальное значение,
    else if (mas[i] > _max) _max = mas[i]; // максимальное
    _avg += mas[i]; // и сумму для вычисления среднего значения
  }
  lcd.clear(); // Очищаем дисплей и выводим результат
  lcd.printf("min=%d, max=%d", _min, _max);
  lcd.printfAt(0, 1, "avg=%d", _avg / 10);
}

void  exampleSelectVal(){
  // Предложим понизить частоту Ардуино, установив значение предделителя  
  char *dividers[] = {"div_1", "div_2", "div_4", "div_8", "div_16"};
  lcd.printMultiline(F("This example will change your Arduino frequency by setting the System Clock Prescaler Register"));
  clock_prescale_set(clock_div_t(lcd.selectVal("Choose clock divider", dividers, 5, clock_prescale_get())));
  lcd.clear();
  lcd.print("Frequency changed");
}

void exampleInputStrVal(){
  char alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  char _name[21] = {0};
  if (lcd.inputStrVal("What's your name?", _name, 12, alphabet))
    lcd.printf("Hello, %s!", _name);
  else
    lcd.print("Input canceled");
}
