#include <Wire.h>
#include <LiquidCrystal_I2C_Ext.h>
LiquidCrystal_I2C_Ext lcd(0x27, 20, 4);

// Пины для энкодера
#define pinCLK 2
#define pinDT  3
#define pinSW  4

int brightness = 50;
int _delay = 10;

// Перечислим значения, используемые в меню для задания связи родитель-потомок
enum {mkBack, mkRoot, mkSetBrightness, mkSetDelay};

// Опишем меню
sMenuItem menu[] = {
  {mkBack, mkRoot, "Options"},
    {mkRoot, mkSetBrightness, NULL}, // Названия этих пунктов
    {mkRoot, mkSetDelay, NULL}      // сгенерируем позже
};

// Определим колчество элементов в меню
uint8_t menuLen = sizeof(menu) / sizeof(sMenuItem);

// Функция поиска индекса элемента меню по его значению key
int getItemIndexByKey(uint8_t key){
  for (uint8_t i = 0; i < menuLen; i++)
    if (menu[i].key == key)
      return i;
  return -1;
}

//Функция формирования названия пункта меню со значением
void updateCaption(uint8_t key, char format[], int value){
  // key - ключ обновляемого пункта меню
  // format - шаблон для сборки названия пункта меню
  // value - значение, добавляемое в название
  uint8_t index = getItemIndexByKey(key);
  char* buf = (char*) malloc(40);
  sprintf(buf, format, value);
  menu[index].caption = (char*) realloc(menu[index].caption, strlen(buf)+1);
  strcpy(menu[index].caption, buf);
  free(buf);
}

void setup() {
  lcd.begin();
  lcd.attachEncoder(pinDT, pinCLK, pinSW);
  updateCaption(mkSetBrightness, "Brightness (%d%%)", brightness);
  updateCaption(mkSetDelay, "Delay (%dms)", _delay);
}
  
void loop() {
  // Показываем меню
  uint8_t selectedMenuItem = lcd.showMenu(menu, menuLen, 1);
  // и анализируем выбранное значение
  if (selectedMenuItem == mkSetBrightness) {
    // Изменение значения brightness
    brightness = lcd.inputVal("Input brightness(%)", 0, 100, brightness, 5);
    // Обновим название пункта меню
    updateCaption(mkSetBrightness, "Brightness (%d%%)", brightness);
  }
  else if (selectedMenuItem == mkSetDelay) {
    // Изменение значения _delay
    _delay = lcd.inputVal("Input delay(ms)", 0, 20, _delay);
    // Обновим название пункта меню
    updateCaption(mkSetDelay, "Delay (%dms)", _delay);
  } 
}
